#ifndef __LD3320_H
#define __LD3320_H

#define MY_VERSION 0

#define MYDELAY1        delayMicroseconds(1)//delay(1)
#define MYDELAY5        delayMicroseconds(10)//delay(5)
#define MYDELAY20       delayMicroseconds(15)//delay(20)
#define MYDELAY_NOP     delayMicroseconds(1)

#include "Arduino.h"

#define u8 unsigned char


// modified for esp8266, wemos d1 mini.
#define SDCK 5      //SPI 时钟信号D1
#define SDO  4      //SPI slave data output, MI数据输出D2
#define SDI  14     //SPI MO数据输入D5
#define SCS  12     //芯片片选信号D6
#define RSTB 2     //复位端口D4
#define IRQ  13     //中断引脚D7


#if 1

extern volatile unsigned &PIN_OUT_SET;
extern volatile unsigned &PIN_OUT_CLEAR;
extern volatile unsigned &PIN_IN;

#define IO00    0x00001
#define IO01    0x00002
#define IO02    0x00004
#define IO03    0x00008
#define IO04    0x00010
#define IO05    0x00020
#define IO12    0x01000
#define IO13    0x02000
#define IO14    0x04000
#define IO15    0x08000

#define SDCK_1 (PIN_OUT_SET=IO05)
#define SDCK_0 (PIN_OUT_CLEAR=IO05)

#define SDI_1 (PIN_OUT_SET=IO14)
#define SDI_0 (PIN_OUT_CLEAR=IO14)

#define SCS_1 (PIN_OUT_SET=IO12)
#define SCS_0 (PIN_OUT_CLEAR=IO12)

#define RSTB_1 (PIN_OUT_SET=IO02)
#define RSTB_0 (PIN_OUT_CLEAR=IO02)

#define SDO_READ (PIN_IN&IO04)

#else

#define SDCK_1 digitalWrite(SDCK, HIGH)
#define SDCK_0 digitalWrite(SDCK, LOW)

#define SDI_1 digitalWrite(SDI, HIGH)
#define SDI_0 digitalWrite(SDI, LOW)

#define SCS_1 digitalWrite(SCS, HIGH)
#define SCS_0 digitalWrite(SCS, LOW)

#define RSTB_1 digitalWrite(RSTB, HIGH)
#define RSTB_0 digitalWrite(RSTB, LOW)

#define SDO_READ (digitalRead(SDO))

#endif


//  以下三个状态定义用来记录程序是在运行ASR识别还是在运行MP3播放
#define LD_MODE_IDLE      0x00
#define LD_MODE_ASR_RUN   0x08
#define LD_MODE_MP3       0x40


//  以下五个状态定义用来记录程序是在运行ASR识别过程中的哪个状态
#define LD_ASR_NONE       0x00  //  表示没有在作ASR识别
#define LD_ASR_RUNING     0x01  //  表示LD3320正在作ASR识别中
#define LD_ASR_FOUNDOK    0x10  //  表示一次识别流程结束后，有一个识别结果
#define LD_ASR_FOUNDZERO  0x11  //  表示一次识别流程结束后，没有识别结果
#define LD_ASR_ERROR      0x31  //  表示一次识别流程中LD3320芯片内部出现不正确的状态

#define LD_ASR_IDLE       0x99  //  idle
#define LD_ASR_RUNING1    0x9A
#define LD_ASR_ERROR1     0x9B
#define LD_ASR_NONE1      0x9C
#define LD_ASR_NONE2      0x9D


#define CLK_IN          24  /* 用户注意修改输入的晶振时钟大小 */
#define LD_PLL_11       (u8)((CLK_IN/2.0)-1)
#define LD_PLL_MP3_19   0x0f
#define LD_PLL_MP3_1B   0x18
#define LD_PLL_MP3_1D   (u8)(((90.0*((LD_PLL_11)+1))/(CLK_IN))-1)

#define LD_PLL_ASR_19   (u8)(CLK_IN*32.0/(LD_PLL_11+1) - 0.51)
#define LD_PLL_ASR_1B   0x48
#define LD_PLL_ASR_1D   0x1f

#define MIC_VOL 0x43   //咪头增益



//识别码客户修改处
#define CODE_CMD   0x00   //该命令码0x00用户不可进行修改。
#define CODE_DMCS  0x01   //代码测试
#define CODE_CSWB  0x02   //测试完毕

#define CODE_1KL1 0x03
#define CODE_1KL2 0x04
#define CODE_1KL3 0x05
#define CODE_1KL4 0x06
                  
#define CODE_2KL1 0x07
#define CODE_2KL2 0x08
#define CODE_2KL3 0x09
#define CODE_2KL4 0x0a
                  
#define CODE_3KL1 0x0b
#define CODE_3KL2 0x0c
#define CODE_3KL3 0x0d
#define CODE_3KL4 0x0e
                  
#define CODE_4KL1 0x0f
#define CODE_4KL2 0x10
#define CODE_4KL3 0x11
#define CODE_4KL4 0x12

#define CODE_5KL1 0x13

class LD3320
{
  public:
    void LD3320_IO_Init(void);
IRAM_ATTR void LD_WriteReg(unsigned char address,unsigned char dataout);
IRAM_ATTR unsigned char LD_ReadReg(unsigned char address);
    void LD_Reset(void);
    void LD_Init_Common(void);
    void LD_Init_ASR(void);
    void LD_AsrStart(void);
    u8 LD_AsrRun(void);
    u8 LD_AsrAddFixed(void);
    u8 RunASR(void);
    u8 LD_GetResult(void);
    u8 LD_Check_ASRBusyFlag_b2(void);
};


#endif
