// tables.h. the tables lib. v.0.1. by ken woo. 20220607.

// AAA --------------------------------------------------------------- class cLinkedTables/double linked.
#if !defined(_TABLES_H_)
#define _TABLES_H_
#define _TABLES_GC_ 64 // use GC(nonzero/64) or not(0). default is off/0 since gc can cause long recursive dealloc operation.

#include<cstdio>


// strings of a table of the tables of the double-linked structure. for easily accessing any table in it.
// *** note!!! every table used here must have the null-string terminated, the "". ***
// the tables structure is as follows,
//
//    | TBL  |                       | TBL  |                       | TBL  |
//    |  1   | <---double linked---> |  2   | <---double linked---> |  3   | <---double linked---> ... , "".
//    |______|                       |______|                       |______|
//
// a single table example,
//    mytable tbl={
//        "tainan city",
//        "roc taiwan",
//        "esp8266 arduino developments, the double-linked tables for accumulate and separate data strings to different tables",
//        "open source",
//        ""
//    };


typedef char *mycstr;           // a c_string.
typedef mycstr mycstrs[];       // a single table.
typedef mycstrs mytable;        // the alt name of a single table.


typedef class cLinkedTables{public:
    mytable *tbl;               // *** note!!! every table used here must have the null-string terminated, the "". ***
    cLinkedTables *prv;         // the parent node.
    cLinkedTables *nxt;         // the child node.

    #if _TABLES_GC_
        // GC. 12-byte, 3k, 3072/12=256 nodes then to delete. however 256 can cause wdt-reset. use 64 might be proper.
        static unsigned gcn;        // garbage collection, GC, currently total nodes.
        static cLinkedTables *gcp;  // GC, the head pointer.
        static cLinkedTables **gcc; // GC, the current storage for new coming.
        public:
        static void GC(cLinkedTables *p){
            if (!gcp) gcp=p;
            else *gcc=p;
            //// gcc=&(p->nxt); // p may not be a single node, so use the following line.
            for (gcc=&(p->nxt); *gcc; gcc=&((*gcc)->nxt)) gcn++; ////
            if (++gcn>=(_TABLES_GC_)){
                *gcc=0;
                delete gcp;
                gcc=0, gcn=0, gcp=0;
            }
        };
        private:
    #endif // _TABLES_GC_

        public:

    cLinkedTables(): tbl(0), prv(0), nxt(0){;};

    void assignTable(mytable t){ // this is weird, but correct. instead, uses "mytable *t" caused forbidden.
                                // assign the table to this object, would be replaced if old one exists.
        ///// tbl=&t;           // this is weird, should be correct but wrong.
        tbl=(mytable*)t;        // this is weird, should be incorrect but correct.
    };

    void addTable(mytable t){   // always adds to the tail with a new.
        cLinkedTables **a, *b;
        for (a=&nxt, b=this; *a; b=*a, a=&((*a)->nxt));
        (*a)=new cLinkedTables;
        (*a)->prv=b;
        (*a)->tbl=(mytable*)t;
    };

    void InsertTable(mytable t, unsigned front=0){ // insert in front of or next to this one with a new. default is next to it.
        cLinkedTables *b;
        b=new cLinkedTables;
        b->tbl=(mytable*)t;
        if (front){
            if (this->prv){
                this->prv->nxt=b;
                b->prv=this->prv;
            }
            this->prv=b;
            b->nxt=this;
        }
        else {
            if (this->nxt){
                this->nxt->prv=b;
                b->nxt=this->nxt;
            }
            this->nxt=b;
            b->prv=this;
        }
    };

    void delTable(){            // remove the present one/itself by GC.
        if (this->prv){
            this->prv->nxt=this->nxt;
        }
        if (this->nxt){
            this->nxt->prv=this->prv;
        }
        this->nxt=0;

        #if _TABLES_GC_
            GC(this);
        #else // _TABLES_GC_
            delete this;
        #endif // _TABLES_GC_

    };

    mytable* getTableInLevel(unsigned which_lvl=0){ // get the table of the level ranges from 0 to n-1. return 0 if no this level table.
                                // also note that it is the relative position.
                                // the return value is weird. uses "mytable" is forbidden.
        cLinkedTables *a=this;
        for (; which_lvl; which_lvl--){
            if (a) a=a->nxt;
            else return 0;
        }
        if (!a) return 0;
        return a->tbl;
    };

    unsigned assignTableToLevel(mytable t, unsigned which_lvl=0){ // assign the table to the level ranges from 0 to n-1.
                                // note that it is the relative position. would be replaced if old one exists.
                                // return 1 if successful 0 if failed.
        cLinkedTables *a=this;
        for (; which_lvl; which_lvl--){
            if (a) a=a->nxt;
            else return 0;
        }
        if (!a) return 0;
        a->tbl=(mytable*)t;
        return 1;
    };

    static unsigned moveToNext(cLinkedTables *&objptr){ // alter this object pointer to point to the next one. return 0 if failed.
        if (!objptr->nxt) return 0;
        objptr=objptr->nxt;
        return 1;
    };

    static unsigned moveToPrev(cLinkedTables *&objptr){ // alter this object pointer to point to the prev one. return 0 if failed.
        if (!objptr->prv) return 0;
        objptr=objptr->prv;
        return 1;
    };

    cLinkedTables* getFirst(){  // return the linked-list first one object.
        cLinkedTables *i;
        for (i=this; i->prv; i=i->prv);
        return i;
    };

    cLinkedTables* getLast(){   // return the linked-list last one object.
        cLinkedTables *i;
        for (i=this; i->nxt; i=i->nxt);
        return i;
    };

    cLinkedTables* getNext(){   // return the next one object against the current one. be care of return 0.
        return this->nxt;
    };

    cLinkedTables* getPrev(){   // return the previous one object against the current one. be care of return 0.
        return this->prv;
    };

    mytable* getNextTable(){    // return the next one table against the current one. be care of return 0.
        if (this->nxt) return this->nxt->tbl;
        return 0;
    };

    mytable* getPrevTable(){    // return the previous one table against the current one. be care of return 0.
        if (this->prv) return this->prv->tbl;
        return 0;
    };

    cLinkedTables* getAbsoluteLevel(unsigned which_lvl){ // get the object which corresponds to the absolute level it is.
                                // return 0 if failed.
        cLinkedTables *a=getFirst();
        for (; which_lvl; which_lvl--){
            if (a) a=a->nxt;
            else return 0;
        }
        return a;
    };

    unsigned countLevels(){     // return the amount number of levels this linked list has.
        cLinkedTables *i=this;
        unsigned j=1;
        for (; i->prv; i=i->prv, j++);
        for (i=this; i->nxt; i=i->nxt, j++);
        return j;
    };

    ~cLinkedTables(){
                                // the present obj is going to destroy and we do nothing to it.
                                // however, the nxt which points to of this object must be destroyed by us.
        if (this->nxt) delete this->nxt;
        //// delete this->nxt; // could simply do like this since delete 0 does not not ok.
    };

    // since these are often used.
    inline void assT(mytable t){assignTable(t);};
    inline void addT(mytable t){addTable(t);};
    inline void insT(mytable t, unsigned rear=1){InsertTable(t, !rear);};
    inline void delT(){delTable();};
    inline mytable* getT(){return this->tbl;};
    inline mytable* nxtT(){return getNextTable();};
    inline mytable* prvT(){return getPrevTable();};
    inline static unsigned mov2Nxt(cLinkedTables *&objptr){return moveToNext(objptr);};
    inline static unsigned mov2Prv(cLinkedTables *&objptr){return moveToPrev(objptr);};
    inline unsigned nums(){return countLevels();};

    void debug_dumpTables(){
        cLinkedTables *a=getFirst();
        int x=0;
        while (a && a->tbl){
            printf("\r\n-------- level %d\r\n", x++);
            for (int i=0; *((*(a->tbl))[i]); i++) printf("[%s]\r\n", (*(a->tbl))[i]);
            a=a->nxt;
        }
    };

    static void debug_dumpTable(mytable t){
        for (int i=0; *(t[i]); i++) printf("[%s]\r\n", t[i]);
    };

} cLinkedTables;

#if _TABLES_GC_
    // should be in .cpp.
    unsigned cLinkedTables::gcn=0;
    cLinkedTables *cLinkedTables::gcp=0;
    cLinkedTables **cLinkedTables::gcc=0;
#endif // _TABLES_GC_

#endif // defined(_TABLES_H_)
// VVV --------------------------------------------------------------- class cLinkedTables/double linked.
